use rhai::{CustomType, ImmutableString, TypeBuilder};

#[derive(Debug, Clone)]
pub struct MouseClickEvent {
	pub down: bool,
	pub element: ImmutableString,
}

impl CustomType for MouseClickEvent {
	fn build(mut builder: TypeBuilder<Self>) {
		builder
			.with_name("MouseClickEvent")
			.with_fn("is_down", |s: &mut Self| s.down)
			.with_fn("element", |s: &mut Self| s.element.clone());
	}
}

#[derive(Debug, Clone)]
pub struct MouseHoverEvent {
	pub enter: bool,
	pub element: ImmutableString,
}

impl CustomType for MouseHoverEvent {
	fn build(mut builder: TypeBuilder<Self>) {
		builder
			.with_name("MouseHoverEvent")
			.with_fn("is_enter", |s: &mut Self| s.enter)
			.with_fn("element", |s: &mut Self| s.element.clone());
	}
}

#[derive(Debug, Clone)]
pub struct PlayerShipStateEvent {}

impl CustomType for PlayerShipStateEvent {
	fn build(mut builder: TypeBuilder<Self>) {
		builder.with_name("PlayerShipStateEvent");
	}
}

#[derive(Debug, Clone)]
pub struct KeyboardEvent {
	pub down: bool,
	pub key: ImmutableString,
}

impl CustomType for KeyboardEvent {
	fn build(mut builder: TypeBuilder<Self>) {
		builder
			.with_name("KeyboardEvent")
			.with_fn("is_down", |s: &mut Self| s.down)
			.with_fn("key", |s: &mut Self| s.key.clone());
	}
}

#[derive(Debug, Clone)]
pub struct ScrollEvent {
	pub val: f32,
}

impl CustomType for ScrollEvent {
	fn build(mut builder: TypeBuilder<Self>) {
		builder
			.with_name("ScrollEvent")
			.with_fn("val", |s: &mut Self| s.val);
	}
}
